@@ -7,7 +7,7 @@ Attean::QueryPlanner - Query planner
 
 =head1 VERSION
 
-This document describes Attean::QueryPlanner version 0.012
+This document describes Attean::QueryPlanner version 0.017
 
 =head1 SYNOPSIS
 
@@ -17,12 +17,19 @@ This document describes Attean::QueryPlanner version 0.012
   my $default_graphs = [ Attean::IRI->new('http://example.org/') ];
   my $plan = $planner->plan_for_algebra( $algebra, $model, $default_graphs );
   my $iter = $plan->evaluate($model);
-  my $iter = $e->evaluate( $model );
+  while (my $result = $iter->next()) {
+    say $result->as_string;
+  }
 
 =head1 DESCRIPTION
 
-The Attean::QueryPlanner class implements a query planner using the
-iterative dynamic programming approach.
+The Attean::QueryPlanner class is a base class implementing common behavior for
+query planners. Subclasses will need to consume or compose the
+L<Attean::API::JoinPlanner> role.
+
+Trivial sub-classes may consume L<Attean::API::NaiveJoinPlanner>, while more
+complex planners may choose to implement complex join planning (e.g.
+L<Attean::IDPQueryPlanner>).
 
 =head1 ATTRIBUTES
 
@@ -34,7 +41,7 @@ use Attean::Algebra;
 use Attean::Plan;
 use Attean::Expression;
 
-package Attean::QueryPlanner 0.012 {
+package Attean::QueryPlanner 0.017 {
 	use Moo;
 	use Encode qw(encode);
 	use Attean::RDF;
@@ -51,6 +58,7 @@ package Attean::QueryPlanner 0.012 {
 
 	with 'Attean::API::QueryPlanner', 'MooX::Log::Any';
 	has 'counter' => (is => 'rw', isa => Int, default => 0);
+	has 'table_threshold'	=> (is => 'rw', isa => Int, default => 10);
 
 =back
 
@@ -103,7 +111,7 @@ the supplied C<< $active_graph >>.
 		my %args			= @_;
 		
 		if ($model->does('Attean::API::CostPlanner')) {
-			my @plans	= $model->plans_for_algebra($algebra, $model, $active_graphs, $default_graphs, @_);
+			my @plans	= $model->plans_for_algebra($algebra, $self, $active_graphs, $default_graphs, %args);
 			if (@plans) {
 				return @plans; # trust that the model knows better than us what plans are best
 			} else {
@@ -118,7 +126,9 @@ the supplied C<< $active_graph >>.
 		
 		my @children	= @{ $algebra->children };
 		my ($child)		= $children[0];
-		if ($algebra->isa('Attean::Algebra::BGP')) {
+		if ($algebra->isa('Attean::Algebra::Query') or $algebra->isa('Attean::Algebra::Update')) {
+			return $self->plans_for_algebra($algebra->child, $model, $active_graphs, $default_graphs, %args);
+		} elsif ($algebra->isa('Attean::Algebra::BGP')) {
 			my $triples	= $algebra->triples;
 			my @triples	= @$triples;
 			my %blanks;
@@ -149,10 +159,10 @@ the supplied C<< $active_graph >>.
 			return @plans;
 		} elsif ($algebra->isa('Attean::Algebra::Join')) {
 			return $self->group_join_plans($model, $active_graphs, $default_graphs, $interesting, map {
-				[$self->plans_for_algebra($_, $model, $active_graphs, $default_graphs, @_)]
+				[$self->plans_for_algebra($_, $model, $active_graphs, $default_graphs, %args)]
 			} @children);
 		} elsif ($algebra->isa('Attean::Algebra::Distinct') or $algebra->isa('Attean::Algebra::Reduced')) {
-			my @plans	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_);
+			my @plans	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args);
 			my @dist;
 			foreach my $p (@plans) {
 				if ($p->distinct) {
@@ -198,7 +208,7 @@ the supplied C<< $active_graph >>.
 			my %exprs	= ($var => $expr);
 		
 			my @plans;
-			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_)) {
+			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args)) {
 				my $distinct	= $plan->distinct;
 				my $ordered		= $plan->ordered;
 				if ($expr->isa('Attean::ValueExpression') and $expr->value->does('Attean::API::Variable')) {
@@ -220,7 +230,7 @@ the supplied C<< $active_graph >>.
 			my @cmps	= @{ $algebra->comparators };
 			my ($exprs, $ascending, $svars)	= $self->_order_by($algebra);
 			my @plans;
-			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, interesting_order => $algebra->comparators, @_)) {
+			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, interesting_order => $algebra->comparators, %args)) {
 				my $distinct	= $plan->distinct;
 
 				if (scalar(@cmps) == 1 and $cmps[0]->expression->isa('Attean::ValueExpression') and $cmps[0]->expression->value->does('Attean::API::Variable')) {
@@ -244,12 +254,12 @@ the supplied C<< $active_graph >>.
 			
 			return @plans;
 		} elsif ($algebra->isa('Attean::Algebra::LeftJoin')) {
-			my $l	= [$self->plans_for_algebra($children[0], $model, $active_graphs, $default_graphs, @_)];
-			my $r	= [$self->plans_for_algebra($children[1], $model, $active_graphs, $default_graphs, @_)];
+			my $l	= [$self->plans_for_algebra($children[0], $model, $active_graphs, $default_graphs, %args)];
+			my $r	= [$self->plans_for_algebra($children[1], $model, $active_graphs, $default_graphs, %args)];
 			return $self->join_plans($model, $active_graphs, $default_graphs, $l, $r, 'left', $algebra->expression);
 		} elsif ($algebra->isa('Attean::Algebra::Minus')) {
-			my $l	= [$self->plans_for_algebra($children[0], $model, $active_graphs, $default_graphs, @_)];
-			my $r	= [$self->plans_for_algebra($children[1], $model, $active_graphs, $default_graphs, @_)];
+			my $l	= [$self->plans_for_algebra($children[0], $model, $active_graphs, $default_graphs, %args)];
+			my $r	= [$self->plans_for_algebra($children[1], $model, $active_graphs, $default_graphs, %args)];
 			return $self->join_plans($model, $active_graphs, $default_graphs, $l, $r, 'minus');
 		} elsif ($algebra->isa('Attean::Algebra::Project')) {
 			my $vars	= $algebra->variables;
@@ -260,12 +270,20 @@ the supplied C<< $active_graph >>.
 				($vars_key eq join(' ', sort @{ $_->in_scope_variables }))
 					? $_ # no-op if plan is already properly-projected
 					: $self->new_projection($_, $distinct, @vars)
-			} $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_);
+			} $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args);
 			return @plans;
 		} elsif ($algebra->isa('Attean::Algebra::Graph')) {
 			my $graph	= $algebra->graph;
 			if ($graph->does('Attean::API::Term')) {
-				return $self->plans_for_algebra($child, $model, $graph, @_);
+				if (my $available = $args{available_graphs}) {
+					# the list of available graphs has been restricted, and this
+					# graph is not available so return an empty table plan.
+					unless (any { $_->equals($graph) } @$available) {
+						my $plan	= Attean::Plan::Table->new( variables => [], rows => [], distinct => 0, ordered => [] );
+						return $plan;
+					}
+				}
+				return $self->plans_for_algebra($child, $model, [$graph], $default_graphs, %args);
 			} else {
 				my $gvar	= $graph->value;
 				my $graphs	= $model->get_graphs;
@@ -274,6 +292,14 @@ the supplied C<< $active_graph >>.
 				$vars{ $gvar }++;
 				my @vars	= keys %vars;
 				
+				my %available;
+				if (my $available = $args{available_graphs}) {
+					foreach my $a (@$available) {
+						$available{ $a->value }++;
+					}
+					$graphs	= $graphs->grep(sub { $available{ $_->value } });
+				}
+				
 				my @branches;
 				my %ignore	= map { $_->value => 1 } @$default_graphs;
 				while (my $graph = $graphs->next) {
@@ -282,7 +308,7 @@ the supplied C<< $active_graph >>.
 					# TODO: rewrite $child pattern here to replace any occurrences of the variable $gvar to $graph
 					my @plans	= map {
 						Attean::Plan::Extend->new(children => [$_], expressions => \%exprs, distinct => 0, ordered => $_->ordered);
-					} $self->plans_for_algebra($child, $model, [$graph], $default_graphs, @_);
+					} $self->plans_for_algebra($child, $model, [$graph], $default_graphs, %args);
 					push(@branches, \@plans);
 				}
 				
@@ -297,8 +323,17 @@ the supplied C<< $active_graph >>.
 			my $rows	= $algebra->rows;
 			my $vars	= $algebra->variables;
 			my @vars	= map { $_->value } @{ $vars };
-			my $plan	= Attean::Plan::Table->new( variables => $vars, rows => $rows, distinct => 0, ordered => [] );
-			return $plan;
+			
+			if (scalar(@$rows) < $self->table_threshold) {
+				return Attean::Plan::Table->new( variables => $vars, rows => $rows, distinct => 0, ordered => [] );
+			} else {
+				my $iter	= Attean::ListIterator->new(
+					item_type => 'Attean::API::Result',
+					variables => \@vars,
+					values => $rows
+				);
+				return Attean::Plan::Iterator->new( iterator => $iter, distinct => 0, ordered => [] );
+			}
 		} elsif ($algebra->isa('Attean::Algebra::Service')) {
 			my $endpoint	= $algebra->endpoint;
 			my $silent		= $algebra->silent;
@@ -310,7 +345,7 @@ the supplied C<< $active_graph >>.
 			my $limit	= $algebra->limit;
 			my $offset	= $algebra->offset;
 			my @plans;
-			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_)) {
+			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args)) {
 				my $vars	= $plan->in_scope_variables;
 				push(@plans, Attean::Plan::Slice->new(children => [$plan], limit => $limit, offset => $offset, distinct => $plan->distinct, ordered => $plan->ordered));
 			}
@@ -318,7 +353,7 @@ the supplied C<< $active_graph >>.
 		} elsif ($algebra->isa('Attean::Algebra::Union')) {
 			# TODO: if both branches are similarly ordered, we can use Attean::Plan::Merge to keep the resulting plan ordered
 			my @vars		= keys %{ { map { map { $_ => 1 } $_->in_scope_variables } @children } };
-			my @plansets	= map { [$self->plans_for_algebra($_, $model, $active_graphs, $default_graphs, @_)] } @children;
+			my @plansets	= map { [$self->plans_for_algebra($_, $model, $active_graphs, $default_graphs, %args)] } @children;
 
 			my @plans;
 			cartesian {
@@ -332,7 +367,7 @@ the supplied C<< $active_graph >>.
 			my @vars		= $algebra->in_scope_variables;
 
 			my @plans;
-			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_)) {
+			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args)) {
 				my $extend		= Attean::Plan::Extend->new(children => [$plan], expressions => \%exprs, distinct => 0, ordered => $plan->ordered);
 				push(@plans, $extend);
 			}
@@ -346,14 +381,14 @@ the supplied C<< $active_graph >>.
 				$exprs{$var}	= $expr;
 			}
 			my @plans;
-			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_)) {
+			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args)) {
 				my $extend		= Attean::Plan::Aggregate->new(children => [$plan], aggregates => \%exprs, groups => $groups, distinct => 0, ordered => []);
 				push(@plans, $extend);
 			}
 			return @plans;
 		} elsif ($algebra->isa('Attean::Algebra::Ask')) {
 			my @plans;
-			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_)) {
+			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args)) {
 				return Attean::Plan::Exists->new(children => [$plan], distinct => 1, ordered => []);
 			}
 			return @plans;
@@ -385,7 +420,7 @@ the supplied C<< $active_graph >>.
 				my @vars	= $algebra->in_scope_variables;
 				
 				my @joins	= $self->group_join_plans($model, $active_graphs, $default_graphs, $interesting, map {
-					[$self->plans_for_algebra($_, $model, $active_graphs, $default_graphs, @_)]
+					[$self->plans_for_algebra($_, $model, $active_graphs, $default_graphs, %args)]
 				} @join);
 				
 				my @plans;
@@ -409,7 +444,7 @@ the supplied C<< $active_graph >>.
 				} else {
 					$a		= Attean::Algebra::Path->new( subject => $begin, path => $child, object => $end );
 				}
-				my @cplans	= $self->plans_for_algebra($a, $model, $active_graphs, $default_graphs, @_);
+				my @cplans	= $self->plans_for_algebra($a, $model, $active_graphs, $default_graphs, %args);
 				my @plans;
 				foreach my $cplan (@cplans) {
 					my $plan	= Attean::Plan::ALPPath->new(
@@ -428,7 +463,7 @@ the supplied C<< $active_graph >>.
 				return @plans;
 			} elsif ($path->isa('Attean::Algebra::ZeroOrOnePath')) {
 				my $a		= Attean::Algebra::Path->new( subject => $s, path => $path->children->[0], object => $o );
-				my @children	= $self->plans_for_algebra($a, $model, $active_graphs, $default_graphs, @_);
+				my @children	= $self->plans_for_algebra($a, $model, $active_graphs, $default_graphs, %args);
 				my @plans;
 				foreach my $plan (@children) {
 					push(@plans, Attean::Plan::ZeroOrOnePath->new(
@@ -445,20 +480,157 @@ the supplied C<< $active_graph >>.
 				die "Cannot simplify property path $path: " . $algebra->as_string;
 			}
 		} elsif ($algebra->isa('Attean::Algebra::Construct')) {
-			my @children	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_);
+			my @children	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args);
 			my @plans;
 			foreach my $plan (@children) {
 				push(@plans, Attean::Plan::Construct->new(triples => $algebra->triples, children => [$plan], distinct => 0, ordered => []));
 			}
 			return @plans;
 		} elsif ($algebra->isa('Attean::Algebra::Describe')) {
-			my @children	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_);
+			my @children	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args);
 			my @plans;
 			foreach my $plan (@children) {
 				push(@plans, Attean::Plan::Describe->new(terms => $algebra->terms, graph => $active_graphs, children => [$plan], distinct => 0, ordered => []));
 			}
 			return @plans;
-		} elsif ($algebra->isa('Attean::Algebra::Group')) {
+		} elsif ($algebra->isa('Attean::Algebra::Clear')) {
+			my $plan_class	= $algebra->drop ? 'Attean::Plan::Drop' : 'Attean::Plan::Clear';
+			my $target	= $algebra->target;
+			if ($target eq 'GRAPH') {
+				return Attean::Plan::Clear->new(graphs => [$algebra->graph]);
+			} else {
+				my %default	= map { $_->value => 1 } @$active_graphs;
+				my $graphs	= $model->get_graphs;
+				my @graphs;
+				while (my $graph = $graphs->next) {
+					if ($target eq 'ALL') {
+						push(@graphs, $graph);
+					} else {
+						if ($target eq 'DEFAULT' and $default{ $graph->value }) {
+							push(@graphs, $graph);
+						} elsif ($target eq 'NAMED' and not $default{ $graph->value }) {
+							push(@graphs, $graph);
+						}
+					}
+				}
+				return $plan_class->new(graphs => \@graphs);
+			}
+		} elsif ($algebra->isa('Attean::Algebra::Add')) {
+			my $triple	= triplepattern(variable('s'), variable('p'), variable('o'));
+			my $child;
+			my $default_source	= 0;
+			if (my $from = $algebra->source) {
+				($child)	= $self->access_plans( $model, $active_graphs, $triple->as_quad_pattern($from) );
+			} else {
+				$default_source++;
+				my $bgp		= Attean::Algebra::BGP->new( triples => [$triple] );
+				($child)	= $self->plans_for_algebra($bgp, $model, $active_graphs, $default_graphs, %args);
+			}
+			
+			my $dest;
+			my $default_dest	= 0;
+			if (my $g = $algebra->destination) {
+				$dest		= $triple->as_quad_pattern($g);
+			} else {
+				$default_dest++;
+				$dest		= $triple->as_quad_pattern($default_graphs->[0]);
+			}
+
+
+			my @plans;
+			my $run_update	= 1;
+			if ($default_dest and $default_source) {
+				$run_update	= 0;
+			} elsif ($default_dest or $default_source) {
+				#
+			} elsif ($algebra->source->equals($algebra->destination)) {
+				$run_update	= 0;
+			}
+			
+			if ($run_update) {
+				if ($algebra->drop_destination) {
+					my @graphs	= $algebra->has_destination ? $algebra->destination : @$default_graphs;
+					unshift(@plans, Attean::Plan::Clear->new(graphs => [@graphs]));
+				}
+			
+				push(@plans, Attean::Plan::TripleTemplateToModelQuadMethod->new(
+					graph		=> $default_graphs->[0],
+					order		=> ['add_quad'],
+					patterns	=> {'add_quad' => [$dest]},
+					children 	=> [$child],
+				));
+			
+				if ($algebra->drop_source) {
+					my @graphs	= $algebra->has_source ? $algebra->source : @$default_graphs;
+					push(@plans, Attean::Plan::Clear->new(graphs => [@graphs]));
+				}
+			}
+			my $plan	= (scalar(@plans) == 1) ? shift(@plans) : Attean::Plan::Sequence->new( children => \@plans );
+			return $plan;
+		} elsif ($algebra->isa('Attean::Algebra::Modify')) {
+			unless ($child) {
+				# This is an INSERT/DELETE DATA algebra with ground data and no pattern
+				$child	= Attean::Algebra::BGP->new( triples => [] );
+			}
+			
+			my $dataset	= $algebra->dataset;
+			my @default	= @{ $dataset->{default} || [] };
+			my @named	= values %{ $dataset->{named} || {} };
+			
+			my @active_graphs	= @$active_graphs;
+			my @default_graphs	= @$default_graphs;
+			
+			if (scalar(@default) or scalar(@named)) {
+				# change the available named graphs
+				# change the active graph(s)
+				@active_graphs	= @default;
+				@default_graphs	= @default;
+				$args{ available_graphs }	= [@named];
+			} else {
+				# no custom dataset
+			}
+			
+			my @children	= $self->plans_for_algebra($child, $model, \@active_graphs, \@default_graphs, %args);
+			my $i	= $algebra->insert;
+			my $d	= $algebra->delete;
+			my %patterns;
+			my @order;
+			if (scalar(@$d)) {
+				push(@order, 'remove_quad');
+				$patterns{ 'remove_quad' }	= $d;
+			}
+			if (scalar(@$i)) {
+				push(@order, 'add_quad');
+				$patterns{ 'add_quad' }	= $i;
+			}
+			return map {
+				Attean::Plan::TripleTemplateToModelQuadMethod->new(
+					graph		=> $default_graphs->[0],
+					order		=> \@order,
+					patterns	=> \%patterns,
+					children 	=> [$_],
+				)
+			} @children;
+		} elsif ($algebra->isa('Attean::Algebra::Load')) {
+			my $pattern		= triplepattern(variable('subject'), variable('predicate'), variable('object'));
+			my $load		= Attean::Plan::Load->new( url => $algebra->url->value, silent => $algebra->silent );
+			my $graph		= $algebra->has_graph ? $algebra->graph : $default_graphs->[0];
+			my $plan		= Attean::Plan::TripleTemplateToModelQuadMethod->new(
+				graph		=> $graph,
+				order		=> ['add_quad'],
+				patterns	=> {'add_quad' => [$pattern]},
+				children 	=> [$load],
+			);
+			return $plan;
+		} elsif ($algebra->isa('Attean::Algebra::Create')) {
+			return Attean::Plan::Sequence->new( children => [] );
+		} elsif ($algebra->isa('Attean::Algebra::Sequence')) {
+			my @plans;
+			foreach my $child (@{ $algebra->children }) {
+				my ($plan)	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args);
+				push(@plans, $plan);
+			}
+			return Attean::Plan::Sequence->new( children => \@plans );
 		}
 		die "Unimplemented algebra evaluation for: " . $algebra->as_string;
 	}
