@@ -7,7 +7,7 @@ Attean::TripleModel - RDF model backed by a set of triple-stores
 
 =head1 VERSION
 
-This document describes Attean::TripleModel version 0.012
+This document describes Attean::TripleModel version 0.017
 
 =head1 SYNOPSIS
 
@@ -41,7 +41,7 @@ objects representing the backing triple-store for that graph.
 
 =cut
 
-package Attean::TripleModel 0.012 {
+package Attean::TripleModel 0.017 {
 	use Moo;
 	use Types::Standard qw(ArrayRef ConsumerOf HashRef);
 	use Scalar::Util qw(reftype blessed);
@@ -166,7 +166,7 @@ L<Attean::API::QuadIterator>.
 		return Attean::ListIterator->new( values => [], item_type => 'Attean::API::Quad' );
 	}
 	
-=item C<< plans_for_algebra( $algebra, $model, $active_graphs, $default_graphs ) >>
+=item C<< plans_for_algebra( $algebra, $planner, $active_graphs, $default_graphs ) >>
 
 Delegates to an underlying store if the active graph is bound to the store,
 and the store consumes Attean::API::CostPlanner.
@@ -176,7 +176,7 @@ and the store consumes Attean::API::CostPlanner.
 	sub plans_for_algebra {
 		my $self	= shift;
 		my $algebra	= shift;
-		my $model	= shift;
+		my $planner	= shift;
 		my $active_graphs	= shift;
 		my $default_graphs	= shift;
 		my @plans;
@@ -184,7 +184,7 @@ and the store consumes Attean::API::CostPlanner.
 			my $graph	= $active_graphs->[0];
 			if (my $store = $self->stores->{ $graph->value }) {
 				if ($store->does('Attean::API::CostPlanner')) {
-					push(@plans, $store->plans_for_algebra($algebra, $model, $active_graphs, $default_graphs));
+					push(@plans, $store->plans_for_algebra($algebra, $planner, $active_graphs, $default_graphs));
 				}
 			}
 		}
@@ -211,7 +211,7 @@ Attempts to delegate to all the underlying stores if that store consumes Attean:
 	}
 }
 
-package Attean::AddativeTripleModelRole 0.012 {
+package Attean::AddativeTripleModelRole 0.017 {
 	use Moo::Role;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(CodeRef);
@@ -275,7 +275,7 @@ Removes the store associated with the given C<< $graph >>.
 }
 
 
-package Attean::MutableTripleModel 0.012 {
+package Attean::MutableTripleModel 0.017 {
 	use Moo;
 	use Types::Standard qw(ArrayRef ConsumerOf HashRef);
 	use Scalar::Util qw(reftype);
@@ -358,7 +358,7 @@ Removes all quads with the given C<< $graph >>.
 	}
 }
 
-package Attean::AddativeTripleModel 0.012 {
+package Attean::AddativeTripleModel 0.017 {
 	use Moo;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(CodeRef);
@@ -368,7 +368,7 @@ package Attean::AddativeTripleModel 0.012 {
 	with 'Attean::AddativeTripleModelRole';
 }
 
-package Attean::AddativeMutableTripleModel 0.012 {
+package Attean::AddativeMutableTripleModel 0.017 {
 	use Moo;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(CodeRef);
