@@ -7,7 +7,7 @@ AtteanX::Parser::SPARQLTSV - SPARQL Results TSV Parser
 
 =head1 VERSION
 
-This document describes AtteanX::Parser::SPARQLTSV version 0.012
+This document describes AtteanX::Parser::SPARQLTSV version 0.017
 
 =head1 SYNOPSIS
 
@@ -35,7 +35,7 @@ This document describes AtteanX::Parser::SPARQLTSV version 0.012
 
 =cut
 
-package AtteanX::Parser::SPARQLTSV 0.012 {
+package AtteanX::Parser::SPARQLTSV 0.017 {
 	use utf8;
 	use Moo;
 	use Attean;
@@ -79,7 +79,7 @@ the data read from the L<IO::Handle> object C<< $fh >>.
 		my $self	= shift;
 		my $fh		= shift;
 		
-		my $parser	= Attean->get_parser('Turtle')->new();
+		my $parser	= Attean->get_parser('Turtle')->new(lazy_iris => $self->lazy_iris);
 
 		my $line	= <$fh>;
 		unless (defined($line)) {
@@ -115,8 +115,9 @@ the data read from the L<IO::Handle> object C<< $fh >>.
 			return Attean::Result->new( bindings => \%binding );
 		};
 		return Attean::CodeIterator->new(
-			generator => $gen,
-			item_type => $self->handled_type->role,
+			generator	=> $gen,
+			item_type	=> $self->handled_type->role,
+			variables	=> \@vars,
 		);
 	}
 
