@@ -4,7 +4,7 @@ AtteanX::Serializer::NTriples - N-Triples Serializer
 
 =head1 VERSION
 
-This document describes AtteanX::Serializer::NTriples version 0.012
+This document describes AtteanX::Serializer::NTriples version 0.017
 
 =head1 SYNOPSIS
 
@@ -14,7 +14,7 @@ This document describes AtteanX::Serializer::NTriples version 0.012
 
 =head1 DESCRIPTION
 
-...
+Serializes triples into the RDF 1.1 N-Triples format.
 
 =head1 ATTRIBUTES
 
@@ -28,12 +28,16 @@ This document describes AtteanX::Serializer::NTriples version 0.012
 
 =over 4
 
+=item C<< serialize_iter_to_io( $fh, $iterator ) >>
+
+=item C<< serialize_iter_to_bytes( $fh ) >>
+
 =cut
 
 use v5.14;
 use warnings;
 
-package AtteanX::Serializer::NTriples 0.012 {
+package AtteanX::Serializer::NTriples 0.017 {
 	use Moo;
 	use Types::Standard qw(Str ArrayRef);
 	use Encode qw(encode);
@@ -69,6 +73,10 @@ __END__
 Please report any bugs or feature requests to through the GitHub web interface
 at L<https://github.com/kasei/perlrdf/issues>.
 
+=head1 SEE ALSO
+
+L<https://www.w3.org/TR/n-triples/>
+
 =head1 AUTHOR
 
 Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
